<?php
require_once "error_object.inc.php";
require_once "parser_global.inc.php";

class ActivityParser extends XMLParserGlobal{

	const type_galaxyview    = "galaxyview";
	const type_scan          = "scan";
	const type_message       = "message";
	const type_manual        = "manual";
	const type_combat_report = "combat_report";
	const type_allypage      = "alliance_page";

	private $activitytable = "";
	private $activity_collection = array();


	function __construct($dbcon,$player_activitytable,$utablename,$universe) {
		//$this->xml_schema = "xml_schema/fleet_movement.xsd"; - no own parser needed so far
		//  call super constructor
		$result = parent::__construct($dbcon,"DUMMY_TABLE_NAME","DUMMY_TABLE_NAME",$utablename,$universe);
		if ($result === false) {
			return false;
		}

		// local setup
		if (trim($player_activitytable) == "") return false;

		$this->activitytable = $player_activitytable;

		return $this;
	}

	/**
	 * Insert all collected activities into database
	 *
	 * @return Boolean
	 */
	public function insert_into_database() {
		if (count($this->activity_collection) == 0) return true;

		$query = "INSERT IGNORE INTO $this->activitytable (player_id, activity_date, hour, minute, weekday, activity_type) VALUES ";

		foreach ($this->activity_collection as $entry) {
			$query .= "(";
			$query .= $entry['player_id'].",";
			$query .= "'".$entry['year']."-";
			$query .= $entry['month']."-";
			$query .= $entry['day']."',";
			$query .= $entry['hour'].",";
			$query .= $entry['minute'].",";
			$query .= $entry['weekday'].",";
			$query .= "'".$entry['type']."'),";
		}

		// remove last comma
		$query = substr($query,0,strlen($query)-1);

		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting activity details");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}

		return true;
	}

	public function collect_manual_activity($player_id, $year, $month, $day, $hour, $minute, $weekday ) {
		$year    = intval($year);
		$month   = intval($month);
		$day     = intval($day);
		$hour    = intval($hour);
		$minute  = intval($minute);
		$weekday = intval($weekday);

		return $this->buffer_activity($year,$month,$day,$hour,$minute,$weekday,$player_id,ActivityParser::type_manual );
	}

	/**
	 * Collect activity information from a galaxyview for one specific user entry.
	 *
	 * @param DOMNode $activity_DOMNode XML node to be parsed
	 * @param int $player_id Galaxytool playerid
	 * @param string $type Activity type
	 * @return Boolean
	 */
	public function collect_activity(DOMElement $activity_DOMNode,$player_id,$type) {
		if ($activity_DOMNode == null) return true; // nothing to do but nothing bad happened

		$year    = intval($activity_DOMNode->getAttribute("year"));
		$month   = intval($activity_DOMNode->getAttribute("month"));
		$day     = intval($activity_DOMNode->getAttribute("day"));
		$hour    = intval($activity_DOMNode->getAttribute("hour"));
		$minute  = intval($activity_DOMNode->getAttribute("minute"));
		$weekday = intval($activity_DOMNode->getAttribute("weekday"));

		return $this->buffer_activity($year,$month,$day,$hour,$minute,$weekday,$player_id,$type);

	}


	/**
	 * Add one activity entry to buffer to be inserted into database later.
	 *
	 * @param int $year
	 * @param int $month
	 * @param int $day
	 * @param int $hour
	 * @param int $minute
	 * @param int $weekday
	 * @param int $player_id
	 * @param String $activity_type
	 * @return Boolean
	 */
	private function buffer_activity($year, $month, $day, $hour, $minute, $weekday, $player_id, $activity_type) {

		if ($year < 2010 || $year > 2100) return false;
		if ($month < 1 || $month > 12)    return false;
		if ($day < 1 || $day > 31)        return false;
		if ($hour < 0 || $hour > 23)      return false;
		if ($minute < 0 || $minute > 59)  return false;
		if ($weekday < 0 || $weekday > 6) return false; // = Sunday
		if ($player_id < 1) return false;
		if (strtotime("$year-$month-$day $hour:$minute") > time() + 24*60*60) return false; //prohibiting activity in the distant future

		if (
			$activity_type != ActivityParser::type_combat_report &&
			$activity_type != ActivityParser::type_galaxyview &&
			$activity_type != ActivityParser::type_manual &&
			$activity_type != ActivityParser::type_message &&
			$activity_type != ActivityParser::type_scan &&
			$activity_type != ActivityParser::type_allypage
		) return false;

		if ($weekday == 0) $weekday = 7;

		if ($minute > 52) {
			$time = strtotime("$year-$month-$day $hour:$minute");
			$time += 10*60; // add 10 minutes to get next full hour (day, month or even year)

			$year   = date("Y",$time);
			$month  = date("m",$time);
			$day    = date("d",$time);
			$hour   = date("H",$time);
			$minute = 0;
		} elseif ($minute < 8) {
			$minute = 0;
		} elseif ($minute >= 8 && $minute < 23) {
			$minute = 15;
		} elseif ($minute >= 23 && $minute < 38) {
			$minute = 30;
		} else {
			$minute = 45;
		}

		$entry['year']      = $year;
		$entry['month']     = $month;
		$entry['day']       = $day;
		$entry['hour']      = $hour;
		$entry['minute']    = $minute;
		$entry['weekday']   = $weekday;
		$entry['player_id'] = $player_id;
		$entry['type']      = $activity_type;

		array_push($this->activity_collection, $entry);

		return true;
	}

}